/**
 * Victor Silva - University of Bridgeport 20 Feb. 1997
 *
 * Use this class by calling HSV.fromRGB to convert a
 * double array containing image pixels into HSV color space.
 *
 * Alternatively call HSV.toRGB to convert from HSV to
 * RGB.
 *
 * HSV and RGB Converter class.  This class allows conversion
 * between RGB and HSV (Hue, Saturation and Value).
 * The conversion algorithm is from Foley and van Dam
 * "Computer Graphics".
 *
 */
package VS;

import java.awt.*;
import java.io.*;
import ipl.*;

public class HSV extends ColorConverter {

public HSV(ProcessPlane pp_) {
	super(pp_);
}

public int[] fromRGB() {
      totalPix = fi.getLength();
      int[] pixArray = pp.pels;

      double max = 0.0;
      double min = 99999.0;
      double H = 0.0;
      double S = 0.0;
      double V = 0.0;
      double delta;
      ColorUtils CU = new ColorUtils();


      for (int i=0; i<totalPix; i++)
      {
         max = fi.max(i);
         min = fi.min(i);

         V = (double)max;

         // Calculate saturation, S. Saturation is 0.0 if r,g,b are
         // all 0
         S = (max != 0) ? (double)((max - min) / max) : (double)0.0;
         if (S == 0)
         {
            // Really undefined.
            H = 0.0;

            fi.r[i] = (float)H;
            fi.g[i] = (float)S;
            fi.b[i] = (float)V;

            pixArray[i] = CU.putRGB((short)H, (short)S, (short)V);
         }
         else
         {
            delta = (double)(max - min);
            if (fi.r[i] == max)
            {
               H = ((double)(fi.g[i] - fi.b[i])) / delta;
            }
            else
            {
               if (fi.g[i] == max)
               {
                  H = 2.0 + ((double)(fi.b[i] - fi.r[i])) / delta;
               }
               else
               {
                  if (fi.b[i] == max)
                  {
                     H = 4.0 + ((double)(fi.r[i] - fi.g[i])) / delta;
                  }
               }
            }

            // Convert hue to degrees.
            H *= 60.0;
            // make sure hue is nonnegative.
            if (H < 0)
            {
               H += 360.0;
            }

            fi.r[i] = (float)H;
            fi.g[i] = (float)S;
            fi.b[i] = (float)V;

            pixArray[i] = CU.putRGB((short)H, (short)S, (short)V);
         }
      }
      return(pixArray);
   }

public int[] toRGB() {
    float r[] = fi.r;
   	float g[] = fi.g;
   	float b[] = fi.b;

     totalPix = r.length;
      double m1, m2;
      double H;
      double R = 0.0;
      double G = 0.0;
      double B = 0.0;


      for (int i=0; i<totalPix; i++)
      {
         if (g[i] == 0)
         	pp.setPixel(i, (int) b[i], (int) b[i], (int) b[i], 255);
         else
         {
            double J = 0.0;
            double F, P, Q, T;

            if (r[i] == 360)
            {
               H = 0.0;
            }
            H = ((double)r[i]) / 60.0;
            J = Math.floor(H);
            F = H - J;
            P = (double)b[i] * (1.0 - (double)g[i]);
            Q = (double)b[i] * (1.0 - (double)g[i] * F);
            T = (double)b[i] * (1.0 - (double)g[i] * (1 - F));

            switch ((int)J)
            {
            case 0:
               R = (double)b[i];
               G = T;
               B = P;
               break;

            case 1:
               R = Q;
               G = (double)b[i];
               B = P;
               break;

            case 2:
               R = P;
               G = (double)b[i];
               B = T;
               break;

            case 3:
               R = P;
               G = Q;
               B = (double)b[i];
               break;

            case 4:
               R = T;
               G = P;
               B = (double)b[i];
               break;

            case 5:
               R = (double)b[i];
               G = P;
               B = Q;
               break;

            default:
               System.out.println("Error in HSV to RGB conversion.");
               break;
            }
            pp.setPixel(i, (int) R, (int) G, (int) B, 255);
         }
      }
      return(pp.pels);
   }

}