package ipl;
import java.awt.*;
import java.awt.image.*;
import java.util.*;
import gui.*;

class FlipFilter extends ImageFilter {
    int width, height;
    boolean flipH, flipV;

    public void setFlip(boolean flipH, boolean flipV) {
        this.flipH = flipH;
        this.flipV = flipV;
    }

    public void setDimensions(int w, int h) {
        super.setDimensions(width = w, height = h);
    }

    public void setProperties(Hashtable props) {
        boolean h = false;
        boolean v = false;
        Object bh = props.get("flipH");
        Object bv = props.get("flipV");
        
        if (bh != null) {
            h = ((Boolean)bh).booleanValue();
        }
        if (bv != null) {
            v = ((Boolean)bv).booleanValue();
        }
        h ^= flipH;
        v ^= flipV;
        props.put("flipH", new Boolean(h));
        props.put("flipV", new Boolean(v));
        consumer.setProperties(props);
    }

    public void setHints(int h) {
        h |= ImageConsumer.RANDOMPIXELORDER;
        h &= ~ImageConsumer.TOPDOWNLEFTRIGHT;
        super.setHints(h);
    }

    public void setColorModel(ColorModel model) {
        super.setColorModel(model);
    }

    public void setPixels(int srcX, int srcY, int srcW, int srcH,
          ColorModel model, byte pixels[], int srcOff, int srcScan) {
        int s = srcOff;
        byte[] tempBuff = new byte[srcW * srcH];

        for (int y=0; y<srcH; y++) {
            for (int x=0; x<srcW; x++) {
                int d = 0;
                if (flipV) {
                    d += (srcH-y-1)*srcW;
                } else {
                    d += y*srcW;
                }
                if (flipH) {
                    d += (srcW-x-1);
                } else {
                    d += x;
                }
                tempBuff[d] = pixels[s];
                s++;
            }
        }
        
        if (flipH) {
            srcX = width - (srcX+srcW);
        }
        if (flipV) {
            srcY = height - (srcY+srcH);
        }
        super.setPixels(srcX, srcY, srcW, srcH, model, tempBuff, 0, srcScan);
    }

    public void setPixels(int srcX, int srcY, int srcW, int srcH,
          ColorModel model, int pixels[], int srcOff, int srcScan) {
        int s = srcOff;
        int[] tempBuff = new int[srcW * srcH];

        for (int y=0; y<srcH; y++) {
            for (int x=0; x<srcW; x++) {
                int d = 0;
                if (flipV) {
                    d += (srcH-y-1)*srcW;
                } else {
                    d += y*srcW;
                }
                if (flipH) {
                    d += (srcW-x-1);
                } else {
                    d += x;
                }
                tempBuff[d] = pixels[s];
                s++;
            }
        }
        if (flipH) {
            srcX = width - (srcX+srcW);
        }
        if (flipV) {
            srcY = height - (srcY+srcH);
        }
        super.setPixels(srcX, srcY, srcW, srcH, model, tempBuff, 0, srcScan);
    }

    public synchronized void imageComplete(int status) {
        super.imageComplete(status);
    }
}
